@@ -1,14 +1,17 @@
 from collections import defaultdict
 from dataclasses import dataclass
-from functools import lru_cache, wraps
+from enum import Enum
+from functools import lru_cache
 from inspect import Parameter, signature
 from typing import (
     Any,
     Awaitable,
     Callable,
+    Collection,
     Dict,
     Iterator,
     Mapping,
+    MutableMapping,
     Optional,
     Sequence,
     Tuple,
@@ -21,15 +24,18 @@ import graphql
 
 from apischema import UndefinedType
 from apischema.aliases import Aliaser
-from apischema.cache import cache
+from apischema.cache import CacheAwareDict, cache
+from apischema.conversions import Conversion
 from apischema.conversions.conversions import AnyConversion, DefaultConversion
 from apischema.deserialization import deserialization_method
+from apischema.methods import method_registerer
 from apischema.objects import ObjectField
+from apischema.ordering import Ordering
 from apischema.schemas import Schema
 from apischema.serialization import (
+    PassThroughOptions,
     SerializationMethod,
     SerializationMethodVisitor,
-    serialize,
 )
 from apischema.serialization.serialized_methods import (
     ErrorHandler,
@@ -38,46 +44,38 @@ from apischema.serialization.serialized_methods import (
     serialized as register_serialized,
 )
 from apischema.types import AnyType, NoneType, Undefined
+from apischema.typing import is_type
 from apischema.utils import (
     awaitable_origin,
     deprecate_kwargs,
     empty_dict,
     get_args2,
     get_origin_or_type2,
+    identity,
     is_async,
     is_union_of,
     keep_annotations,
-    method_registerer,
 )
 from apischema.validation.errors import ValidationError
 
 
 class PartialSerializationMethodVisitor(SerializationMethodVisitor):
+    use_cache = False
+
     @property
-    def _any_method(self) -> Callable[[type], SerializationMethod]:
-        return partial_serialization_method_factory(
-            self.aliaser, self._conversions, self.default_conversion
-        )
+    def _factory(self) -> Callable[[type], SerializationMethod]:
+        return lambda _: identity
+
+    def enum(self, cls: Type[Enum]) -> SerializationMethod:
+        return identity
 
     def object(self, tp: AnyType, fields: Sequence[ObjectField]) -> SerializationMethod:
-        return lambda obj: obj
+        return identity
 
-    def union(self, alternatives: Sequence[AnyType]) -> SerializationMethod:
-        method = super().union([alt for alt in alternatives if alt != UndefinedType])
-        if UndefinedType in alternatives:
-
-            @wraps(method)
-            def wrapper(obj: Any) -> Any:
-                return obj if obj is Undefined else method(obj)
-
-            return wrapper
-        else:
-            return method
-
-    def unsupported(self, tp: AnyType) -> SerializationMethod:
+    def visit(self, tp: AnyType) -> SerializationMethod:
         if tp is UndefinedType:
             return lambda obj: None
-        return super().unsupported(tp)
+        return super().visit(tp)
 
 
 @cache
@@ -89,7 +87,15 @@ def partial_serialization_method_factory(
     @lru_cache()
     def factory(tp: AnyType) -> SerializationMethod:
         return PartialSerializationMethodVisitor(
-            aliaser, False, False, default_conversion, False, False
+            additional_properties=False,
+            aliaser=aliaser,
+            check_type=False,
+            default_conversion=default_conversion,
+            exclude_defaults=False,
+            exclude_none=False,
+            exclude_unset=False,
+            fall_back_on_any=False,
+            pass_through_options=PassThroughOptions(),
         ).visit_with_conv(tp, conversion)
 
     return factory
@@ -114,10 +120,12 @@ class Resolver(SerializedMethod):
         return super().return_type(unwrap_awaitable(return_type))
 
 
-_resolvers: Dict[Type, Dict[str, Resolver]] = defaultdict(dict)
+_resolvers: MutableMapping[Type, Dict[str, Resolver]] = CacheAwareDict(
+    defaultdict(dict)
+)
 
 
-def get_resolvers(tp: AnyType) -> Mapping[str, Tuple[Resolver, Mapping[str, AnyType]]]:
+def get_resolvers(tp: AnyType) -> Collection[Tuple[Resolver, Mapping[str, AnyType]]]:
     return _get_methods(tp, _resolvers)
 
 
@@ -154,8 +162,9 @@ def resolver(
     alias: str = None,
     *,
     conversion: AnyConversion = None,
-    schema: Schema = None,
     error_handler: ErrorHandler = Undefined,
+    order: Optional[Ordering] = None,
+    schema: Schema = None,
     parameters_metadata: Mapping[str, Mapping] = None,
     serialized: bool = False,
     owner: Type = None,
@@ -169,8 +178,9 @@ def resolver(
     *,
     alias: str = None,
     conversion: AnyConversion = None,
-    schema: Schema = None,
     error_handler: ErrorHandler = Undefined,
+    order: Optional[Ordering] = None,
+    schema: Schema = None,
     parameters_metadata: Mapping[str, Mapping] = None,
     serialized: bool = False,
     owner: Type = None,
@@ -185,9 +195,11 @@ def resolver(
             error_handler2 = None
         resolver = Resolver(
             func,
+            alias2,
             conversion,
-            schema,
             error_handler2,
+            order,
+            schema,
             parameters,
             parameters_metadata or {},
         )
@@ -231,6 +243,12 @@ def resolver_resolve(
     default_serialization: DefaultConversion,
     serialized: bool = True,
 ) -> Callable:
+    # graphql deserialization will give Enum objects instead of strings
+    def handle_enum(tp: AnyType) -> Optional[AnyConversion]:
+        if is_type(tp) and issubclass(tp, Enum):
+            return Conversion(identity, source=Any, target=tp)
+        return default_deserialization(tp)
+
     parameters, info_parameter = [], None
     for param in resolver.parameters:
         param_type = types[param.name]
@@ -250,7 +268,7 @@ def resolver_resolve(
                 aliaser=aliaser,
                 coerce=False,
                 conversion=param_field.deserialization,
-                default_conversion=default_deserialization,
+                default_conversion=handle_enum,
                 fall_back_on_default=False,
                 schema=param_field.schema,
             )
@@ -271,7 +289,7 @@ def resolver_resolve(
 
     serialize_result: Callable[[Any], Any]
     if not serialized:
-        serialize_result = lambda res: res
+        serialize_result = identity
     elif is_async(resolver.func):
         serialize_result = as_async(method_factory(types["return"]))
     else:
@@ -304,11 +322,7 @@ def resolver_resolve(
                 values[param_name] = None
 
         if errors:
-            raise TypeError(
-                serialize(
-                    ValidationError, ValidationError(children=errors), aliaser=aliaser
-                )
-            )
+            raise ValueError(ValidationError(children=errors).errors)
         if info_parameter:
             values[info_parameter] = __info
         try:
