@@ -1,6 +1,6 @@
 from contextlib import suppress
+from dataclasses import dataclass
 from enum import Enum
-from functools import reduce
 from itertools import chain
 from typing import (
     AbstractSet,
@@ -9,7 +9,6 @@ from typing import (
     ClassVar,
     Collection,
     Dict,
-    Iterable,
     List,
     Mapping,
     Optional,
@@ -44,28 +43,54 @@ from apischema.json_schema.refs import (
 from apischema.json_schema.types import JsonSchema, JsonType, json_schema
 from apischema.json_schema.versions import JsonSchemaVersion, RefFactory
 from apischema.metadata.keys import SCHEMA_METADATA
-from apischema.objects import ObjectField
+from apischema.objects import AliasedStr, ObjectField
 from apischema.objects.visitor import (
     DeserializationObjectVisitor,
     ObjectVisitor,
     SerializationObjectVisitor,
 )
-from apischema.schemas import Schema, get_schema
+from apischema.ordering import Ordering, sort_by_order
+from apischema.schemas import Schema, get_schema as _get_schema, merge_schema
 from apischema.serialization import serialize
-from apischema.serialization.serialized_methods import get_serialized_methods
+from apischema.serialization.serialized_methods import (
+    SerializedMethod,
+    get_serialized_methods,
+)
 from apischema.type_names import TypeNameFactory, get_type_name
 from apischema.types import AnyType, OrderedDict, UndefinedType
-from apischema.typing import get_args
+from apischema.typing import get_args, is_typed_dict
 from apischema.utils import (
     context_setter,
     deprecate_kwargs,
     get_origin_or_type,
     is_union_of,
     literal_values,
-    sort_by_annotations_position,
 )
 
 
+def get_schema(tp: AnyType) -> Optional[Schema]:
+    from apischema import settings
+
+    return merge_schema(settings.base_schema.type(tp), _get_schema(tp))
+
+
+def get_field_schema(tp: AnyType, field: ObjectField) -> Optional[Schema]:
+    from apischema import settings
+
+    assert not field.is_aggregate
+    return merge_schema(
+        settings.base_schema.field(tp, field.name, field.alias), field.schema
+    )
+
+
+def get_method_schema(tp: AnyType, method: SerializedMethod) -> Optional[Schema]:
+    from apischema import settings
+
+    return merge_schema(
+        settings.base_schema.method(tp, method.func, method.alias), method.schema
+    )
+
+
 def full_schema(base_schema: JsonSchema, schema: Optional[Schema]) -> JsonSchema:
     if schema is not None:
         base_schema = JsonSchema(base_schema)
@@ -76,6 +101,15 @@ def full_schema(base_schema: JsonSchema, schema: Optional[Schema]) -> JsonSchema
 Method = TypeVar("Method", bound=Callable)
 
 
+@dataclass(frozen=True)
+class Property:
+    alias: AliasedStr
+    name: str
+    ordering: Optional[Ordering]
+    required: bool
+    schema: JsonSchema
+
+
 class SchemaBuilder(
     ConversionsVisitor[Conv, JsonSchema],
     ObjectVisitor[JsonSchema],
@@ -84,7 +118,6 @@ class SchemaBuilder(
     def __init__(
         self,
         additional_properties: bool,
-        aliaser: Aliaser,
         default_conversion: DefaultConversion,
         ignore_first_ref: bool,
         ref_factory: RefFactory,
@@ -92,7 +125,6 @@ class SchemaBuilder(
     ):
         super().__init__(default_conversion)
         self.additional_properties = additional_properties
-        self.aliaser = aliaser
         self._ignore_first_ref = ignore_first_ref
         self.ref_factory = ref_factory
         self.refs = refs
@@ -108,23 +140,21 @@ class SchemaBuilder(
             return JsonSchema({"$ref": self.ref_factory(ref)})
 
     def annotated(self, tp: AnyType, annotations: Sequence[Any]) -> JsonSchema:
-        schemas: List[Optional[Schema]] = []
+        schema = None
         for annotation in reversed(annotations):
             if isinstance(annotation, TypeNameFactory):
                 ref = annotation.to_type_name(tp).json_schema
                 ref_schema = self.ref_schema(ref)
                 if ref_schema is not None:
-                    return reduce(full_schema, reversed(schemas), ref_schema)
+                    return full_schema(ref_schema, schema)
             if isinstance(annotation, Mapping):
-                schemas.append(annotation.get(SCHEMA_METADATA))
-        return reduce(
-            full_schema, reversed(schemas), super().annotated(tp, annotations)
-        )
+                schema = merge_schema(annotation.get(SCHEMA_METADATA), schema)
+        return full_schema(super().annotated(tp, annotations), schema)
 
     def any(self) -> JsonSchema:
         return JsonSchema()
 
-    def collection(self, cls: Type[Iterable], value_type: AnyType) -> JsonSchema:
+    def collection(self, cls: Type[Collection], value_type: AnyType) -> JsonSchema:
         return json_schema(
             type=JsonType.ARRAY,
             items=self.visit(value_type),
@@ -163,18 +193,15 @@ class SchemaBuilder(
         else:
             return json_schema(type=JsonType.OBJECT, additionalProperties=value)
 
-    def visit_field(self, field: ObjectField) -> JsonSchema:
+    def visit_field(
+        self, tp: AnyType, field: ObjectField, required: bool = True
+    ) -> JsonSchema:
+        assert not field.is_aggregate
         result = full_schema(
             self.visit_with_conv(field.type, self._field_conversion(field)),
-            field.schema,
+            get_field_schema(tp, field) if tp is not None else field.schema,
         )
-        if (
-            not field.flattened
-            and not field.pattern_properties
-            and not field.additional_properties
-            and not field.required
-            and "default" not in result
-        ):
+        if not required and "default" not in result:
             result = JsonSchema(result)
             with suppress(Exception):
                 result["default"] = serialize(
@@ -186,71 +213,80 @@ class SchemaBuilder(
                 )
         return result
 
-    def _properties_schema(self, field: ObjectField) -> JsonSchema:
-        assert field.pattern_properties is not None or field.additional_properties
+    def _object_schema(self, cls: type, field: ObjectField) -> JsonSchema:
+        assert field.is_aggregate
         with context_setter(self):
             self._ignore_first_ref = True
-            props_schema = self.visit_field(field)
-        if not props_schema.get("type") == JsonType.OBJECT:
-            raise TypeError("properties field must have an 'object' type")
-        if "patternProperties" in props_schema:
-            if (
-                len(props_schema["patternProperties"]) != 1
-                or "additionalProperties" in props_schema
-            ):  # don't try to merge the schemas
-                pass
-            else:
-                return next(iter(props_schema["patternProperties"].values()))
-        elif "additionalProperties" in props_schema:
-            if isinstance(props_schema["additionalProperties"], JsonSchema):
-                return props_schema["additionalProperties"]
-            else:  # there is maybe only properties
-                pass
+            object_schema = full_schema(
+                self.visit_with_conv(field.type, self._field_conversion(field)),
+                field.schema,
+            )
+        if object_schema.get("type") not in {JsonType.OBJECT, "object"}:
+            field_type = "Flattened" if field.flattened else "Properties"
+            raise TypeError(
+                f"{field_type} field {cls.__name__}.{field.name}"
+                f" must have an object type"
+            )
+        return object_schema
+
+    def _properties_schema(
+        self, object_schema: JsonSchema, pattern: Optional[Pattern] = None
+    ):
+        if "patternProperties" in object_schema:
+            if pattern is not None:
+                for p in (pattern, pattern.pattern):
+                    if p in object_schema["patternProperties"]:
+                        return object_schema["patternProperties"][p]
+            elif (
+                len(object_schema["patternProperties"]) == 1
+                and "additionalProperties" not in object_schema
+            ):
+                return next(iter(object_schema["patternProperties"].values()))
+        if isinstance(object_schema.get("additionalProperties"), Mapping):
+            return object_schema["additionalProperties"]
         return JsonSchema()
 
-    def _check_flattened_schema(self, cls: Type, field: ObjectField):
-        assert field.flattened
-        with context_setter(self):
-            self._ignore_first_ref = True
-            if self.visit_field(field).get("type") not in {JsonType.OBJECT, "object"}:
-                raise TypeError(
-                    f"Flattened field {cls.__name__}.{field.name} must have an object type"
-                )
-
-    @staticmethod
-    def _field_required(field: ObjectField):
-        return field.required
+    def properties(
+        self, tp: AnyType, fields: Sequence[ObjectField]
+    ) -> Sequence[Property]:
+        raise NotImplementedError
 
     def object(self, tp: AnyType, fields: Sequence[ObjectField]) -> JsonSchema:
         cls = get_origin_or_type(tp)
+        properties = sort_by_order(
+            cls, self.properties(tp, fields), lambda p: p.name, lambda p: p.ordering
+        )
         flattened_schemas: List[JsonSchema] = []
         pattern_properties = {}
         additional_properties: Union[bool, JsonSchema] = self.additional_properties
-        properties = {}
-        required = []
         for field in fields:
             if field.flattened:
-                self._check_flattened_schema(cls, field)
-                flattened_schemas.append(self.visit_field(field))
+                self._object_schema(cls, field)  # check the field is an object
+                flattened_schemas.append(
+                    full_schema(
+                        self.visit_with_conv(field.type, self._field_conversion(field)),
+                        field.schema,
+                    )
+                )
             elif field.pattern_properties is not None:
                 if field.pattern_properties is ...:
                     pattern = infer_pattern(field.type, self.default_conversion)
                 else:
                     assert isinstance(field.pattern_properties, Pattern)
                     pattern = field.pattern_properties
-                pattern_properties[pattern] = self._properties_schema(field)
+                pattern_properties[pattern] = self._properties_schema(
+                    self._object_schema(cls, field), pattern
+                )
             elif field.additional_properties:
-                additional_properties = self._properties_schema(field)
-            else:
-                properties[self.aliaser(field.alias)] = self.visit_field(field)
-                if self._field_required(field):
-                    required.append(field.alias)
+                additional_properties = self._properties_schema(
+                    self._object_schema(cls, field)
+                )
         alias_by_names = {f.name: f.alias for f in fields}.__getitem__
         dependent_required = get_dependent_required(cls)
         result = json_schema(
             type=JsonType.OBJECT,
-            properties=properties,
-            required=required,
+            properties={p.alias: p.schema for p in properties},
+            required=[p.alias for p in properties if p.required],
             additionalProperties=additional_properties,
             patternProperties=pattern_properties,
             dependentRequired=OrderedDict(
@@ -272,13 +308,13 @@ class SchemaBuilder(
     def tuple(self, types: Sequence[AnyType]) -> JsonSchema:
         return json_schema(
             type=JsonType.ARRAY,
-            items=[self.visit(cls) for cls in types],
+            prefixItems=[self.visit(cls) for cls in types],
+            items=False,
             minItems=len(types),
             maxItems=len(types),
         )
 
-    def _union_result(self, results: Iterable[JsonSchema]) -> JsonSchema:
-        results = list(results)
+    def _visited_union(self, results: Sequence[JsonSchema]) -> JsonSchema:
         if len(results) == 1:
             return results[0]
         elif any(alt == {} for alt in results):
@@ -309,9 +345,6 @@ class SchemaBuilder(
         else:
             return json_schema(anyOf=results)
 
-    def union(self, alternatives: Sequence[AnyType]) -> JsonSchema:
-        return super().union([alt for alt in alternatives if alt is not UndefinedType])
-
     def visit_conversion(
         self,
         tp: AnyType,
@@ -319,17 +352,17 @@ class SchemaBuilder(
         dynamic: bool,
         next_conversion: Optional[AnyConversion] = None,
     ) -> JsonSchema:
-        schemas = []
+        schema = None
         if not dynamic:
             for ref_tp in self.resolve_conversion(tp):
                 ref_schema = self.ref_schema(get_type_name(ref_tp).json_schema)
                 if ref_schema is not None:
                     return ref_schema
             if get_args(tp):
-                schemas.append(get_schema(get_origin_or_type(tp)))
-            schemas.append(get_schema(tp))
+                schema = merge_schema(schema, get_schema(get_origin_or_type(tp)))
+            schema = merge_schema(schema, get_schema(tp))
         result = super().visit_conversion(tp, conversion, dynamic, next_conversion)
-        return reduce(full_schema, schemas, result)
+        return full_schema(result, schema)
 
     RefsExtractor: ClassVar[Type[RefsExtractor_]]
 
@@ -341,6 +374,21 @@ class DeserializationSchemaBuilder(
 ):
     RefsExtractor = DeserializationRefsExtractor
 
+    def properties(
+        self, tp: AnyType, fields: Sequence[ObjectField]
+    ) -> Sequence[Property]:
+        return [
+            Property(
+                AliasedStr(field.alias),
+                field.name,
+                field.ordering,
+                field.required,
+                self.visit_field(tp, field, field.required),
+            )
+            for field in fields
+            if not field.is_aggregate
+        ]
+
 
 class SerializationSchemaBuilder(
     SchemaBuilder[Serialization],
@@ -351,36 +399,48 @@ class SerializationSchemaBuilder(
 
     @staticmethod
     def _field_required(field: ObjectField):
-        return field.required and not is_union_of(field.type, UndefinedType)
+        from apischema import settings
 
-    def object(self, tp: AnyType, fields: Sequence[ObjectField]) -> JsonSchema:
-        result = super().object(tp, fields)
-        name_by_aliases = {f.alias: f.name for f in fields}
-        properties = {}
-        required = []
-        for alias, (serialized, types) in get_serialized_methods(tp).items():
-            return_type = types["return"]
-            properties[self.aliaser(alias)] = full_schema(
-                self.visit_with_conv(return_type, serialized.conversion),
-                serialized.schema,
+        return not field.skippable(
+            settings.serialization.exclude_defaults, settings.serialization.exclude_none
+        )
+
+    def properties(
+        self, tp: AnyType, fields: Sequence[ObjectField]
+    ) -> Sequence[Property]:
+        from apischema import settings
+
+        return [
+            Property(
+                AliasedStr(field.alias),
+                field.name,
+                field.ordering,
+                required,
+                self.visit_field(tp, field, required),
             )
-            if not is_union_of(return_type, UndefinedType):
-                required.append(alias)
-            name_by_aliases[alias] = serialized.func.__name__
-        if "allOf" not in result:
-            to_update = result
-        else:
-            to_update = result["allOf"][0]
-        if required:
-            required.extend(to_update.get("required", ()))
-            to_update["required"] = sorted(required)
-        if properties:
-            properties.update(to_update.get("properties", {}))
-            props = sort_by_annotations_position(
-                get_origin_or_type(tp), properties, lambda p: name_by_aliases[p]
+            for field in fields
+            if not field.is_aggregate
+            for required in [
+                field.required
+                if is_typed_dict(get_origin_or_type(tp))
+                else not field.skippable(
+                    settings.serialization.exclude_defaults,
+                    settings.serialization.exclude_none,
+                )
+            ]
+        ] + [
+            Property(
+                AliasedStr(serialized.alias),
+                serialized.func.__name__,
+                serialized.ordering,
+                not is_union_of(types["return"], UndefinedType),
+                full_schema(
+                    self.visit_with_conv(types["return"], serialized.conversion),
+                    get_method_schema(tp, serialized),
+                ),
             )
-            to_update["properties"] = {p: properties[p] for p in props}
-        return result
+            for serialized, types in get_serialized_methods(tp)
+        ]
 
 
 TypesWithConversion = Collection[Union[AnyType, Tuple[AnyType, AnyConversion]]]
@@ -420,7 +480,6 @@ def _extract_refs(
 
 def _refs_schema(
     builder: Type[SchemaBuilder],
-    aliaser: Aliaser,
     default_conversion: DefaultConversion,
     refs: Mapping[str, AnyType],
     ref_factory: RefFactory,
@@ -428,7 +487,7 @@ def _refs_schema(
 ) -> Mapping[str, JsonSchema]:
     return {
         ref: builder(
-            additional_properties, aliaser, default_conversion, True, ref_factory, refs
+            additional_properties, default_conversion, True, ref_factory, refs
         ).visit(tp)
         for ref, tp in refs.items()
     }
@@ -453,21 +512,16 @@ def _schema(
     if aliaser is None:
         aliaser = settings.aliaser
     if additional_properties is None:
-        additional_properties = settings.deserialization.additional_properties
+        additional_properties = settings.additional_properties
     version, ref_factory, all_refs = _default_version(version, ref_factory, all_refs)
     refs = _extract_refs([(tp, conversion)], default_conversion, builder, all_refs)
     json_schema = builder(
-        additional_properties, aliaser, default_conversion, False, ref_factory, refs
+        additional_properties, default_conversion, False, ref_factory, refs
     ).visit_with_conv(tp, conversion)
     json_schema = full_schema(json_schema, schema)
-    if add_defs:
+    if add_defs and version.defs:
         defs = _refs_schema(
-            builder,
-            aliaser,
-            default_conversion,
-            refs,
-            ref_factory,
-            additional_properties,
+            builder, default_conversion, refs, ref_factory, additional_properties
         )
         if defs:
             json_schema["$defs"] = defs
@@ -475,10 +529,10 @@ def _schema(
         JsonSchema,
         json_schema,
         aliaser=aliaser,
-        fall_back_on_any=True,
         check_type=True,
         conversion=version.conversion,
         default_conversion=converters.default_serialization,
+        fall_back_on_any=True,
     )
     if with_schema and version.schema is not None:
         result["$schema"] = version.schema
@@ -520,15 +574,15 @@ def deserialization_schema(
 def serialization_schema(
     tp: AnyType,
     *,
-    schema: Schema = None,
+    additional_properties: bool = None,
+    all_refs: bool = None,
+    aliaser: Aliaser = None,
     conversion: AnyConversion = None,
     default_conversion: DefaultConversion = None,
-    version: JsonSchemaVersion = None,
-    aliaser: Aliaser = None,
     ref_factory: RefFactory = None,
-    all_refs: bool = None,
+    schema: Schema = None,
+    version: JsonSchemaVersion = None,
     with_schema: bool = True,
-    additional_properties: bool = None,
 ) -> Mapping[str, Any]:
     from apischema import settings
 
@@ -551,14 +605,12 @@ def _defs_schema(
     types: TypesWithConversion,
     default_conversion: DefaultConversion,
     builder: Type[SchemaBuilder],
-    aliaser: Aliaser,
     ref_factory: RefFactory,
     all_refs: bool,
     additional_properties: bool,
 ) -> Mapping[str, JsonSchema]:
     return _refs_schema(
         builder,
-        aliaser,
         default_conversion,
         _extract_refs(types, default_conversion, builder, all_refs),
         ref_factory,
@@ -631,7 +683,7 @@ def definitions_schema(
     from apischema import settings
 
     if additional_properties is None:
-        additional_properties = settings.deserialization.additional_properties
+        additional_properties = settings.additional_properties
     if aliaser is None:
         aliaser = settings.aliaser
     if default_deserialization is None:
@@ -643,7 +695,6 @@ def definitions_schema(
         deserialization,
         default_deserialization,
         DeserializationSchemaBuilder,
-        aliaser,
         ref_factory,
         all_refs,
         additional_properties,
@@ -652,7 +703,6 @@ def definitions_schema(
         serialization,
         default_serialization,
         SerializationSchemaBuilder,
-        aliaser,
         ref_factory,
         all_refs,
         additional_properties,
